#include "TextureShader.h"
#include "gtc/type_ptr.hpp"

using namespace glm;

TextureShader::TextureShader(const std::string& path, vec3 amb_col, vec3 spec_col, float shin)
: Shader("./Shader/TextureShader.vert", "./Shader/TextureShader.frag")
//: Shader("./Shader/TextureShader.vert", "./Shader/BlinnPhongeShader.frag")
, texture(new Texture(path))
{
	
	useShader();

	projected_texture0 = new Texture("./Textures/evilmaster01rahmen.jpg");
	projected_texture1 = new Texture("./Textures/spider1.jpg");
	projected_texture2 = new Texture("./Textures/window.jpg");
	projected_texture3 = new Texture("./Textures/vampire.jpg");
	projected_texture4 = new Texture("./Textures/clown.jpg");
	//projected_texture = new Texture("./Textures/MichaelWimmer.jpg");


	// Set material specific uniforms
	auto amb_col_location = glGetUniformLocation(programHandle, "material.ambient_color");
	auto spec_col_location = glGetUniformLocation(programHandle, "material.specular_color");
	auto shin_location = glGetUniformLocation(programHandle, "material.shininess");
	glUniform3fv(amb_col_location, 1, glm::value_ptr(amb_col));
	glUniform3fv(spec_col_location, 1, glm::value_ptr(spec_col));
	glUniform1f(shin_location, shin);

	glUseProgram(0);

}

TextureShader::~TextureShader() {
	delete texture; texture = nullptr;
	delete projected_texture0; projected_texture0 = nullptr;
	delete projected_texture1; projected_texture1 = nullptr;
	delete projected_texture2; projected_texture2 = nullptr;
	delete projected_texture3; projected_texture3 = nullptr;
	delete projected_texture4; projected_texture4 = nullptr;
}

void TextureShader::bindTexture(int unit) {
	useShader();

	bindTexture(texture, unit, "colorTexture", false);
	bindTexture(projected_texture0, unit + 1, "projectorMap0", true);
	bindTexture(projected_texture1, unit + 6, "projectorMap1", true);
	bindTexture(projected_texture2, unit + 3, "projectorMap2", true);
	bindTexture(projected_texture3, unit + 4, "projectorMap3", true);
	bindTexture(projected_texture4, unit + 5, "projectorMap4", true);
}
void TextureShader::bindTexture(Texture* textureToBind, int unit, char* uniformName, bool clampToBorder) {
	useShader();

	textureToBind->bind(unit);
	if (clampToBorder){ // do not repeat/mirror texture, make everything outside [0,1] transparent
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_R, GL_CLAMP_TO_BORDER);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
		float color[] = { 0.0f, 0.0f, 0.0f, 0.0f };
		glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, color);
	}
	else{ // repeat

	}
	if (useMipMapping == 0){ //use no mipmapping
		if (useLinearTexSampling){
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		}
		else{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		}
	}
	else if (useMipMapping == 1){ //use nearest neighbour mipmapping
		if (useLinearTexSampling){
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
		}
		else{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_NEAREST);
		}
	}
	else{
		if (useLinearTexSampling){ //use linear sampling mipmapping
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
		}
		else{
			glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST_MIPMAP_LINEAR);
		}
	}
	auto tex_location = glGetUniformLocation(programHandle, uniformName);
	glUniform1i(tex_location, unit);
}

void TextureShader::setLighting(std::vector<std::shared_ptr<PointLight>> *allLights, int sm_light_id) {
	useShader(); 
	
	glm::vec3 att[20];
	glm::vec3 intensity[20];
	glm::vec3 location[20];
	glm::vec3 on[25];

	for (int i = 0; i < allLights->size(); i++){
		att[i] = vec3(allLights->at(i)->const_attenuation, allLights->at(i)->lin_attenuation, allLights->at(i)->quad_attenuation);
		location[i] = allLights->at(i)->location();
		intensity[i] = allLights->at(i)->intensity*vec3(1.1, 0.9, 0.8);
		if (allLights->at(i)->on){
			on[i] = glm::vec3(1,1,1);
		}
		else{
			on[i] = glm::vec3(0, 0, 0);
		}
	}
	for (int i = allLights->size(); i < 20; i++){
		att[i] = vec3(0,0,0);
		location[i] = vec3(0, 0, 0);
		intensity[i] = vec3(0, 0, 0);
		on[i] = glm::vec3(1, 1, 1);
	}
	//for (int i = 20; i < 25; i++){
	//	//for (int i = allLights->size(); i < 25; i++){
	//	att[i] = vec3(0, 0, 0);
	//	location[i] = vec3(0, 0, 0);
	//	intensity[i] = vec3(0, 0, 0);
	//	//on[i] = false;
	//	if (projected_textur->at(i)->on){
	//		on[i] = glm::vec3(1, 1, 1);
	//	}
	//	else{
	//		on[i] = glm::vec3(0, 0, 0);
	//	}
	//}
	on[20] = glm::vec3(0, 0, 0);
	on[21] = glm::vec3(0, 0, 0);
	on[22] = glm::vec3(0, 0, 0);
	on[23] = glm::vec3(0, 0, 0);
	on[24] = glm::vec3(0, 0, 0);


	auto light_count_location = glGetUniformLocation(programHandle, "light_count");
	auto light_loc_location = glGetUniformLocation(programHandle, "light_location");
	auto light_int_location = glGetUniformLocation(programHandle, "light_intensity");
	auto light_att_location = glGetUniformLocation(programHandle, "light_attenuation");
	auto light_on_location = glGetUniformLocation(programHandle, "light_on");
	glUniform1i(light_count_location, allLights->size()); 
	glUniform3fv(light_loc_location, 20, glm::value_ptr(location[0]));
	glUniform3fv(light_int_location, 20, glm::value_ptr(intensity[0]));
	glUniform3fv(light_att_location, 20, glm::value_ptr(att[0]));
	//glUniform1iv(light_on_location, 25, (GLint*) &on[0]);
	glUniform3fv(light_on_location, 25, glm::value_ptr(on[0]));
	// Shadow map uniforms
	auto id_loc = glGetUniformLocation(programHandle, "sm_light_id");
	auto pos_loc = glGetUniformLocation(programHandle, "sm_light_pos");
	glUniform1i(id_loc, sm_light_id);
	glUniform3fv(pos_loc, 1, glm::value_ptr(location[sm_light_id]));

	glUseProgram(0);
}